;*******************************************************************************************
;   Beschreibung:	Schrittmotorsteuerung (mit L298 Leistungsstufe.	16-fach Microschritt und							 * 
;					Vollschritt	ber PMW. 16MHZ Oscillator																								 *							 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	SMPMW-P16F1825.asm						                               		 *
;    Datum:        	09.03.2024                                                            *
;    Version: 		V1.0                                                                  *
;                                                                                          *
;    Autor:       	Werner Hirsch                                                       *
;    Internet:      www.wernerhirsch.de                                                           *
;    e-mail:		wernerhirschhh@aol.com                                                          *
;                                                                                          *
;*******************************************************************************************
;
;
;
;
;																										 *
;													 *
;                                                                                          *
;*******************************************************************************************


	list      p=16f1825		;definiert den verwendeten Prozessor
	#include <p16f1825.inc>		;in diesem File stehen die Adressen und Bitnamen

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES
	
   __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_OFF & _BORV_19 & _LVP_OFF
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus


;===========================
;  DEFINITIONEN
;===========================

#define IN_CLK			PORTA, 0		;Eingang Clock liegt an PIN RA0 (pin 13), PGD
#define IN_VREF			PORTA, 1		;Vref+ liegt an PIN RA1 (pin 12), PGC
#define IN_ENABLE		PORTA, 2		;Eingang Enable liegt an PIN RA2 (pin 11), 0=off, 1=on
#define IN_NCA3			PORTA, 3		;PIN RA3 (pin 4),Vpp
#define IN_SENSE_1		PORTA, 4		;Eingang Sense 1 liegt an PIN RA4 (AN3)(pin 3)
#define OUT_2A			PORTA, 5		;Ausgang 2A liegt an PIN RA5 (pin 2)
#define IN_RL			PORTC, 0		;Eingang Rechts / Links liegt an PIN RC0 (pin 10), Links=0, Rechts=1
#define IN_SPEED		PORTC, 1		;Eingang Vollschritt/ Microschritt liegt an PIN RC1 (pin 9), Voll=0, Micro=1
#define OUT_2B			PORTC, 2		;Ausgang 2B liegt an PIN RC2 (pin 8)
#define IN_SENSE_2		PORTC, 3		;Eingang Sense 2 liegt an PIN RC3 (AN7)(pin 7)
#define OUT_1B			PORTC, 4		;Ausgang 1B liegt an PIN RC4 (pin 6)
#define OUT_1A			PORTC, 5		;Ausgang 1A liegt an PIN RC5 (pin 5)


;===========================
;  KONSTANTEN
;===========================

SOLL_MAX		EQU 0x40		;Maximaler Sollwert fr Vollschritt = 64 dezimal

;===========================
;  VARIABLEN
;===========================

COUNT_FULL		EQU 0x20		;Zhlregister fr Takt Vollschritt
COUNT_CLK		EQU 0x21		;Zhlregister fr Takt Mikroschritt
STAT_CLK		EQU 0x22		;Status Eingangstakt
STAT_ENABLE		EQU 0x23		;Status Eingang Enable
PMWREG_1		EQU 0x24		;Sollwert Spannung fr PMW 1
PMWREG_2		EQU 0x25		;Sollwert Spannung fr PMW 2
PMWOUT_1		EQU 0x26		;Register fr Ausgang PMW 1
PMWOUT_2		EQU 0x27		;Register fr Ausgang PMW 2
SOLL_1			EQU 0x28		;
SOLL_2			EQU 0x29		;
RECHNEN			EQU 0x2A		;Allgemeines Rechenregister
SPEED			EQU 0x2B		;Register fr Vollschritt = 1, Microschritt = 0
TEMP_01			EQU 0x2C		;
TEMP_02			EQU 0x2D		;


;__________________________________________________________________________________________

		ORG     0x0000			; Adresse, an die nach einem Reset gesprungen wird
		PAGESEL Start
  		goto    Start           ; Springe zum Programmanfang

		ORG     0x0004          ; Adresse, an die nach einem Interrupt gesprungen wird

		; Ab hier kann der Code fr die Interruptroutinen stehen


		retfie                    	; Rcksprung vom Interrupt
;__________________________________________________________________________________________

Start						; Beginn des Programms
	; Initialisierungen
	BANKSEL PORTA			;Select BANK 0
	clrf PORTA				;Lsche alle Ausgangsports
	clrf PORTC

	BANKSEL LATA			;Select BANK 2
	clrf LATA
	clrf LATC

	movlw b'00000001'
	movwf APFCON1			;Bit0: 0 = P2A auf RC3,  1 = P2A auf RA5,  Bit2: 0 = P2B auf RC2,  1 = P2B auf RA4,  

	BANKSEL ANSELA			;Select BANK 3
	movlw b'00010010'		;Pin RA1, RA4, sind analoge Eingnge (Vref+, AN3)
	movwf ANSELA
	movlw b'00001000'		;Pin RC3 ist analoger Eingang (AN7)
	movwf ANSELC			;

	BANKSEL TRISA			;Select BANK 1
	movlw b'01111010'
	movwf OSCCON			;Interner Oszillator, 16 MHz
	movlw b'00000000'     	;PortB pull-up on
	movwf OPTION_REG 
	movlw b'00000001'		;ADC enabled
	movwf ADCON0			;
	movlw b'01010010'		;bit 7=0: linksbndig, bit 6-4=101: f/16, bit 1-0:+Vref extern
	movwf ADCON1

	movlw b'11011111'		;setze RA0 bis RA4 als Eingang, RA5 als Ausgang
	movwf TRISA
	movlw b'11001011'		;setze RC0, RC1 und RC3 als Eingang, RC2 und RC4 bis RC5 als Ausgang
	movwf TRISC

	BANKSEL WPUA			;Select BANK 4
	movlw b'00001101'		;Pull Up: RA0, RA2, RA3
	movwf WPUA
	movlw b'00000011'		;Pull Up: RC0, RC1
	movwf WPUC
	
	BANKSEL PORTA			;Select BANK 0
							; PWM vorbereiten; Vorteiler 1:1 und Timer2 einschalten
	bcf	T2CON,T2CKPS0		; Vorteiler 1:1 (bit 0 = 0)
	bcf	T2CON,T2CKPS1		; Vorteiler 1:1	(bit 1 = 0)
	bsf	T2CON,TMR2ON		; Timer2 ein

							; Frequenz auf 15,6 kHz einstellen: 16 MHz / 4 / (255+1) = 15,6 KHz
	movlw D'255'			;
	movwf PR2				; 15,6 kHz


	BANKSEL CCPR1L			;Select BANK 5
							; Tastverhltnis auf 0% einstellen
	clrf CCPR1L				; 0%
	clrf CCPR2L				; 0%
	clrf CCPR1H
	clrf CCPR2H
							; PWM MODE mit CCP1+2 initialisieren
	clrf CCPTMRS
	movlw b'00001100'		;CCPx-Modus PWM-Mode: Single, PxA,PxB aktive High
	movwf CCP1CON
	movwf CCP2CON
	movlw b'00010001'		;Bits 1-0: 01: A ist aktiv, B inaktiv,  10: A ist inaktiv, B aktiv
	movwf PSTR1CON
	movlw b'00010010'		;Bits 1-0: 01: A ist aktiv, B inaktiv,  10: A ist inaktiv, B aktiv
	movwf PSTR2CON

	BANKSEL PORTA			;Select BANK 0
	bcf PIR1,CCP1IF			;Lsche Interrupt Flags
	bcf PIR2,CCP2IF

	clrf SPEED				;Lsche Register
	clrf STAT_CLK			;
	call PWM_disable

;Testbereich_Anfang

;Testbereich_Ende

Main
	btfsc IN_ENABLE		; Prfen ob das Eingangssignal Enable low ist
	goto $+3			; Enable ist high, weiter bei Prfung des Taktsignals
	call PWM_disable	; Enable ist low, 
	goto ADC_PMW_1		;

	btfsc STAT_ENABLE,0	;
	goto Clock			;
	bsf STAT_ENABLE,0	;
	goto Sollwerte

Clock
	btfsc IN_CLK		; Prfen ob das Taktsignal low ist
	goto $+3			; wenn ja berspringe diesen Befehl
	clrf STAT_CLK		; wenn nein lsche bit in Status register
	goto ADC_PMW_1		;
	btfsc STAT_CLK,0
	goto ADC_PMW_1		;
	bsf STAT_CLK,0		;

Takt
	clrf SPEED			; lsche Register
	btfss IN_SPEED		; Prfen ob das Geschwindigkeitssignal high ist
	bsf SPEED,0			; Geschwindigkeitssignal ist low, (=> Vollschritt) setze Register auf 1
	btfss IN_RL			; Prfen ob das Rechts/Links Signal high ist (=>rechts)
	goto Takt_links		; Signal ist low, gehe zu Takt_links

Takt_rechts
	btfsc SPEED,0		; Prfen ob das Geschwindigkeitsregister  0 ist (=>Mikroschritt)
	goto Takt_R_V		; Geschwindigkeitsregister  =1 (=> Vollschritt), gehe zu Takt Rechts Vollschritt
	incf COUNT_CLK		; erhhe Zhler um 1
	movlw d'16'			; Schreibe Wert 16 ins W-Register
	subwf COUNT_CLK,W	; subtrahiere Zhler fr Mikroschritt im W-Register
	btfss STATUS,Z		; Prfen, ob Ergebnis 0 ist
	goto Sollwerte		; Ergebniss >0<, weiter bei Sollwerten
	clrf COUNT_CLK		; Ergebniss =0, rcksetzen des Zhlers auf 0

Takt_R_V
	incf COUNT_FULL		; erhhe Zhler um 1
	movlw d'4'			; Schreibe Wert 4 ins W-Register
	subwf COUNT_FULL, W	; subtrahiere Zhler fr Vollschritt im W-Register
	btfss STATUS, Z		; Prfen, ob Ergebnis 0 ist
	goto Sollwerte		; Ergebniss >0<, weiter bei Sollwerten
	clrf COUNT_FULL		; Ergebniss =0, rcksetzen des Zhlers auf 0
	goto Sollwerte

Takt_links
	btfsc SPEED, 0		; Prfen ob das Geschwindigkeitsregister  0 ist (=>Mikroschritt)
	goto Takt_L_V		; Geschwindigkeitsregister  =1 (=> Vollschritt), gehe zu Takt Links Vollschritt
	movfw COUNT_CLK		; hole Zhler fr Mikroschritt ins W-Register
	btfsc STATUS, Z		; Prfen, ob Ergebnis >0< ist
	goto $+3			; Ergebnis ist 0, rcksetzen des Zhlers fr Microschritt auf 15
	decf COUNT_CLK		; Ergebniss >0<, reduziere Zhler um 1
	goto Sollwerte		;
	movlw d'15'			; Schreibe Wert 15 ins W-Register
	movwf COUNT_CLK		; rcksetzen des Zhlers fr Microschritt auf 15

Takt_L_V
	movfw COUNT_FULL	; hole Zhler fr Vollschritt ins W-Register
	btfsc STATUS, Z		; Prfen, ob Ergebnis 0 ist
	goto $+3			; Ergebnis ist 0, rcksetzen des Zhlers fr Vollschritt auf 3
	decf COUNT_FULL		; Ergebniss >0<, reduziere Zhler um 1
	goto Sollwerte
	movlw d'3'			; Schreibe Wert 3 ins W-Register
	movwf COUNT_FULL	; rcksetzen des Zhlers fr Vollschritt auf 3
	goto Sollwerte	

Sollwerte
	clrf PCLATH			;Programmzhler fuer folgende Tabelle im Bereich 0x0000..0x00FF
	movfw COUNT_FULL	;
	addwf PCL,f
	goto Takt_V1		;
	goto Takt_V2		;
	goto Takt_V3		;
	goto Takt_V4		;
Takt_V1
	BANKSEL CCPR1L		;Select BANK 5; PSTRxCON: Bits 1-0: 01: A ist aktiv, B inaktiv,  10: A ist inaktiv, B aktiv
	movlw b'00010001'	;1A aktiv, 1B inaktiv
	movwf PSTR1CON		;
	movlw b'00010010'	;2A inaktiv, 2B aktiv
	movwf PSTR2CON	
	BANKSEL PORTA		;Select BANK 0
	btfss SPEED, 0		; Prfen ob das Geschwindigkeitssignal high ist
	goto microstep_1	; wenn ja berspringe diesen Befehl, wenn nein gehe zu microstep_1
	movlw SOLL_MAX		; hole Max Sollwert ins W-Register
	movwf PMWREG_1		; lade Sollwert in PMWREG_1+2 Register
	movwf PMWREG_2		;
	goto ADC_PMW_1		;
Takt_V2
	BANKSEL CCPR1L		;Select BANK 5; PSTRxCON: Bits 1-0: 01: A ist aktiv, B inaktiv,  10: A ist inaktiv, B aktiv
	movlw b'00010001'	;1A aktiv, 1B inaktiv
	movwf PSTR1CON		;
	movlw b'00010001'	;2A aktiv, 2B inaktiv
	movwf PSTR2CON
	BANKSEL PORTA		;Select BANK 0
	btfss SPEED, 0		; Prfen ob das Geschwindigkeitssignal high ist
	goto microstep_2	; wenn ja berspringe diesen Befehl, wenn nein gehe zu microstep_2
	movlw SOLL_MAX		; hole Max Sollwert ins W-Register
	movwf PMWREG_1		; lade Sollwert in PMWREG_1+2 Register
	movwf PMWREG_2		;
	goto ADC_PMW_1		;
Takt_V3
	BANKSEL CCPR1L		;Select BANK 5; PSTRxCON: Bits 1-0: 01: A ist aktiv, B inaktiv,  10: A ist inaktiv, B aktiv
	movlw b'00010010'	;1A inaktiv, 1B aktiv
	movwf PSTR1CON		;
	movlw b'00010001'	;2A aktiv, 2B inaktiv
	movwf PSTR2CON
	BANKSEL PORTA		;Select BANK 0
	btfss SPEED, 0		; Prfen ob das Geschwindigkeitssignal high ist
	goto microstep_1	; wenn ja berspringe diesen Befehl, wenn nein gehe zu microstep_1
	movlw SOLL_MAX		; hole Max Sollwert ins W-Register
	movwf PMWREG_1		; lade Sollwert in PMWREG_1+2 Register
	movwf PMWREG_2		;
	goto ADC_PMW_1		;
Takt_V4
	BANKSEL CCPR1L		;Select BANK 5; PSTRxCON: Bits 1-0: 01: A ist aktiv, B inaktiv,  10: A ist inaktiv, B aktiv
	movlw b'00010010'	;1A inaktiv, 1B aktiv
	movwf PSTR1CON		;
	movlw b'00010010'	;2A inaktiv, 2B aktiv
	movwf PSTR2CON	
	BANKSEL PORTA		;Select BANK 0
	btfss SPEED, 0		; Prfen ob das Geschwindigkeitssignal high ist
	goto microstep_2	; wenn ja berspringe diesen Befehl, wenn nein gehe zu microstep_2
	movlw SOLL_MAX		; hole Max Sollwert ins W-Register
	movwf PMWREG_1		; lade Sollwert in PMWREG_1+2 Register
	movwf PMWREG_2		;
	goto ADC_PMW_1		;

microstep_1
	movlw 0x07
	movwf PCLATH		;Programmzhler fuer Sinus Tabelle im Bereich 0x0700..0x071F
	movfw COUNT_CLK		;hole Wert des Mikroschrittzhlers nach W
	call sin_tab		;Aufruf der Sinustabelle im Bereich 0x0700...0x071F
	movwf PMWREG_1		;
	movfw COUNT_CLK
	addlw d'16'
	call sin_tab
	movwf PMWREG_2		;
	goto ADC_PMW_1		;

microstep_2
	movlw 0x07
	movwf PCLATH		;Programmzhler fuer Sinus Tabelle im Bereich 0x0700..0x071F
	movfw COUNT_CLK		;hole Wert des Mikroschrittzhlers nach W
	call sin_tab		;Aufruf der Sinustabelle im Bereich 0x0700...0x071F
	movwf PMWREG_2		;
	movfw COUNT_CLK
	addlw d'16'
	call sin_tab
	movwf PMWREG_1		;
	goto ADC_PMW_1		;


ADC_PMW_1				; Eingangsspannung wandeln
	movfw PMWREG_1		; hole Sollwert ins W-Register
	addwf SOLL_1, W		; Alten Sollwert addieren
	movwf RECHNEN		;
	bcf STATUS, C		; Carrybit lschen, damit es beim rotieren nicht strt
	rrf RECHNEN, F		; teile durch 2
	movfw RECHNEN		;
	movwf SOLL_1
 
	BANKSEL TRISA		;Select BANK 1
	movlw b'00001101'	;ADC-Eingang AN3 auswhlen, CHS=00011, (RA4)
	movwf ADCON0		;
	call Messen			; ADC starten und obere 8 Bit auslesen

	BANKSEL PORTA		;Select BANK 0
	subwf SOLL_1, W		; Sollwert subtrahieren
	btfss STATUS, C		; prfen ob das ergebnis negativ ist (carry bit = 1)
	goto PMW_1_dec		; wenn ja berspringe diesen Befehl, wenn nein gehe zu PMW_1_dec
	btfss STATUS, Z		; prfen ob das Ergebnis 0 ist (zero bit = 1)
	goto PMW_1_inc		; wenn ja berspringe diesen Befehl, wenn nein gehe zu PMW_1_inc
	goto ADC_PMW_2		; gehe zu Continue_1

PMW_1_dec
	movfw PMWOUT_1		; hole PMWOUT_1 ins W-Register
	btfss STATUS, Z		; Prfung ob Wert =0 (zero bit = 1)
	decf PMWOUT_1		; wenn ja berspringe diesen Befehl, wenn nein, reduziere Wert um 1
	goto ADC_PMW_2		; weiter bei ADC_PMW_2

PMW_1_inc
	movlw d'255'		; 
	subwf PMWOUT_1, W	; subtrahiere PMWOUT_1
	btfss STATUS, Z		; Prfung ob Ergebnis =0 (zero bit = 1)
	incf PMWOUT_1		; wenn ja berspringe diesen Befehl, wenn nein, erhhe Wert um 1
	goto ADC_PMW_2		; weiter bei ADC_PMW_2

ADC_PMW_2
	movfw PMWREG_2		; hole Sollwert ins W-Register
	addwf SOLL_2, W		; Alten Sollwert addieren
	movwf RECHNEN		;
	bcf STATUS, C		; Carrybit lschen, damit es beim rotieren nicht strt
	rrf RECHNEN, F		; teile durch 2
	movfw RECHNEN		;
	movwf SOLL_2

	BANKSEL TRISA		;Select BANK 1
	movlw b'00011101'	;ADC-Eingang AN7 auswhlen, CHS=00111, (RC3)
	movwf ADCON0		;
	call Messen			; ADC starten und obere 8 Bit auslesen

	BANKSEL PORTA		;Select BANK 0
	subwf SOLL_2, W		; Sollwert subtrahieren
	btfss STATUS, C		; prfen ob das ergebnis negativ ist (carry bit = 1)
	goto PMW_2_dec		; wenn ja berspringe diesen Befehl, wenn nein gehe zu PMW_2_dec
	btfss STATUS, Z		; prfen ob das Ergebnis 0 ist (zero bit = 1)
	goto PMW_2_inc		; wenn ja berspringe diesen Befehl, wenn nein gehe zu PMW_2_inc
	goto PMW_OUT		; gehe zu PMW_OUT

PMW_2_dec
	movfw PMWOUT_2		; hole PMWOUT_2 ins W-Register
	btfss STATUS, Z		; Prfung ob Wert =0 (zero bit = 1)
	decf PMWOUT_2		; wenn ja berspringe diesen Befehl, wenn nein, reduziere Wert um 1
	goto PMW_OUT		; weiter bei PMW_OUT

PMW_2_inc
	movlw d'255'		; 
	subwf PMWOUT_2, W	; subtrahiere PMWOUT_2
	btfss STATUS, Z		; Prfung ob Ergebnis =0 (zero bit = 1)
	incf PMWOUT_2		; wenn ja berspringe diesen Befehl, wenn nein, erhhe Wert um 1
	goto PMW_OUT		; weiter bei PMW_OUT


PMW_OUT
	movfw PMWOUT_1		; Wert an PWM bergeben
	BANKSEL CCPR1L		;Select BANK 5
	movwf CCPR1L		; 
	BANKSEL PORTA		;Select BANK 0
	movfw PMWOUT_2		; Wert an PWM bergeben
	BANKSEL CCPR1L		;Select BANK 5
	movwf CCPR2L		; 
	BANKSEL PORTA		;Select BANK 0
	goto Main	

Messen					; Unterprogramm fr Spannungsmessung an AN0 und AN1
	bsf	ADCON0,1		; ADC starten
loop
	btfsc ADCON0,1		; prfen ob der ADC fertig ist?
	goto loop			; nein, weiter warten
	movfw ADRESH		; obere 8 Bit auslesen
	return

PWM_disable
	clrf STAT_ENABLE	;
	clrf PMWREG_1		; Enable ist low, d.h. disabled, setze PWM Solwertregister auf 0
	clrf PMWREG_2
	clrf SOLL_1
	clrf SOLL_2
	clrf PMWOUT_1
	clrf PMWOUT_2
	clrf COUNT_FULL
	clrf COUNT_CLK
	return


	ORG     0x06FF		;

sin_tab
	addwf PCL,f

	retlw d'6'			;DP 0
	retlw d'12'			;DP 1
	retlw d'18'			;DP 2
	retlw d'24'			;DP 3
	retlw d'30'			;DP 4
	retlw d'35'			;DP 5
	retlw d'40'			;DP 6
	retlw d'45'			;DP 7
	retlw d'49'			;DP 8
	retlw d'52'			;DP 9
	retlw d'56'			;DP 10
	retlw d'58'			;DP 11
	retlw d'60'			;DP 12
	retlw d'62'			;DP 13
	retlw d'63'			;DP 14
	retlw d'63'			;DP 15
	retlw d'63'			;DP 16
	retlw d'62'			;DP 17
	retlw d'60'			;DP 18
	retlw d'58'			;DP 19
	retlw d'56'			;DP 20
	retlw d'52'			;DP 21
	retlw d'49'			;DP 22
	retlw d'45'			;DP 23
	retlw d'40'			;DP 24
	retlw d'35'			;DP 25
	retlw d'30'			;DP 26
	retlw d'24'			;DP 27
	retlw d'18'			;DP 28
	retlw d'12'			;DP 29
	retlw d'6'			;DP 30
	retlw d'0'			;DP 31

	END                	; Ende des Programms

